'use strict';

var eff = require('@eslint-react/eff');
var typescript = require('typescript');
var z = require('@zod/mini');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var z__namespace = /*#__PURE__*/_interopNamespace(z);

var __defProp = Object.defineProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// src/JsxConfig/index.ts
var JsxConfig_exports = {};
__export(JsxConfig_exports, {
  getFromAnnotation: () => getFromAnnotation,
  getFromContext: () => getFromContext,
  make: () => make
});

// src/RE.ts
var RE_exports = {};
__export(RE_exports, {
  ANNOTATION_JSX: () => ANNOTATION_JSX,
  ANNOTATION_JSX_FRAG: () => ANNOTATION_JSX_FRAG,
  ANNOTATION_JSX_IMPORT_SOURCE: () => ANNOTATION_JSX_IMPORT_SOURCE,
  ANNOTATION_JSX_RUNTIME: () => ANNOTATION_JSX_RUNTIME,
  CAMEL_CASE: () => CAMEL_CASE,
  COMPONENT_NAME: () => COMPONENT_NAME,
  COMPONENT_NAME_LOOSE: () => COMPONENT_NAME_LOOSE,
  CONSTANT_CASE: () => CONSTANT_CASE,
  HOOK_NAME: () => HOOK_NAME,
  JAVASCRIPT_PROTOCOL: () => JAVASCRIPT_PROTOCOL,
  JS_EXT: () => JS_EXT,
  JS_IDENTIFIER: () => JS_IDENTIFIER,
  KEBAB_CASE: () => KEBAB_CASE,
  PASCAL_CASE: () => PASCAL_CASE,
  REGEXP_STR: () => REGEXP_STR,
  SNAKE_CASE: () => SNAKE_CASE,
  TS_EXT: () => TS_EXT,
  isRegExp: () => isRegExp,
  toRegExp: () => toRegExp
});
var TS_EXT = /^[cm]?tsx?$/u;
var JS_EXT = /^[cm]?jsx?$/u;
var PASCAL_CASE = /^[A-Z][\dA-Za-z]*$/u;
var CAMEL_CASE = /^[a-z][\dA-Za-z]*$/u;
var KEBAB_CASE = /^[a-z][\d\-a-z]*$/u;
var SNAKE_CASE = /^[a-z][\d_a-z]*$/u;
var CONSTANT_CASE = /^[A-Z][\d_A-Z]*$/u;
var JAVASCRIPT_PROTOCOL = /^[\u0000-\u001F ]*j[\t\n\r]*a[\t\n\r]*v[\t\n\r]*a[\t\n\r]*s[\t\n\r]*c[\t\n\r]*r[\t\n\r]*i[\t\n\r]*p[\t\n\r]*t[\t\n\r]*:/iu;
var JS_IDENTIFIER = /^[_$a-z][\w$]*$/i;
var REGEXP_STR = /^\/(.+)\/([A-Za-z]*)$/u;
var ANNOTATION_JSX = /@jsx\s+(\S+)/u;
var ANNOTATION_JSX_FRAG = /@jsxFrag\s+(\S+)/u;
var ANNOTATION_JSX_RUNTIME = /@jsxRuntime\s+(\S+)/u;
var ANNOTATION_JSX_IMPORT_SOURCE = /@jsxImportSource\s+(\S+)/u;
var COMPONENT_NAME = /^[A-Z]/u;
var COMPONENT_NAME_LOOSE = /^_?[A-Z]/u;
var HOOK_NAME = /^use/u;
function toRegExp(string) {
  const [, pattern, flags = "u"] = REGEXP_STR.exec(string) ?? [];
  if (pattern != null) return new RegExp(pattern, flags);
  return { test: (s) => s === string };
}
function isRegExp(string) {
  return Boolean(REGEXP_STR.test(string));
}

// src/JsxConfig/JsxConfig.ts
function make() {
  return {};
}
function getFromContext(context) {
  const options = context.sourceCode.parserServices?.program?.getCompilerOptions() ?? {};
  return {
    jsx: options.jsx ?? typescript.JsxEmit.ReactJSX,
    jsxFactory: options.jsxFactory ?? "React.createElement",
    jsxFragmentFactory: options.jsxFragmentFactory ?? "React.Fragment",
    jsxImportSource: options.jsxImportSource ?? "react",
    reactNamespace: options.reactNamespace ?? "React"
  };
}
var cache = /* @__PURE__ */ new WeakMap();
function getFromAnnotation(context) {
  return eff.getOrElseUpdate(
    cache,
    context.sourceCode,
    () => {
      const options = make();
      if (!context.sourceCode.text.includes("@jsx")) return options;
      let jsx, jsxFrag, jsxRuntime, jsxImportSource;
      for (const comment of context.sourceCode.getAllComments().reverse()) {
        const value = comment.value;
        jsx ??= value.match(ANNOTATION_JSX)?.[1];
        jsxFrag ??= value.match(ANNOTATION_JSX_FRAG)?.[1];
        jsxRuntime ??= value.match(ANNOTATION_JSX_RUNTIME)?.[1];
        jsxImportSource ??= value.match(ANNOTATION_JSX_IMPORT_SOURCE)?.[1];
      }
      if (jsx != null) options.jsxFactory = jsx;
      if (jsxFrag != null) options.jsxFragmentFactory = jsxFrag;
      if (jsxRuntime != null) options.jsx = jsxRuntime === "classic" ? typescript.JsxEmit.React : typescript.JsxEmit.ReactJSX;
      if (jsxImportSource != null) options.jsxImportSource = jsxImportSource;
      return options;
    }
  );
}

// src/LanguagePreference/index.ts
var LanguagePreference_exports = {};
__export(LanguagePreference_exports, {
  LanguagePreferenceSchema: () => LanguagePreferenceSchema,
  getFromContext: () => getFromContext2,
  make: () => make2
});

// src/LanguagePreference/LanguagePreference.ts
function make2() {
  return {
    indentStyle: "space",
    indentWidth: 2,
    jsxQuoteStyle: "double",
    quoteStyle: "single",
    semicolons: "always",
    trailingCommas: "all"
  };
}
function getFromContext2() {
  throw new Error("getFromContext is not implemented");
}
var LanguagePreferenceSchema = z__namespace.object({
  indentStyle: z__namespace.optional(
    z__namespace.union([
      z__namespace.literal("tab"),
      z__namespace.literal("space")
    ])
  ),
  indentWidth: z__namespace.optional(z__namespace.number()),
  quoteStyle: z__namespace.optional(
    z__namespace.union([
      z__namespace.literal("single"),
      z__namespace.literal("double")
    ])
  ),
  semicolons: z__namespace.optional(
    z__namespace.union([
      z__namespace.literal("always"),
      z__namespace.literal("asNeeded")
    ])
  ),
  trailingCommas: z__namespace.optional(
    z__namespace.union([
      z__namespace.literal("all"),
      z__namespace.literal("es5"),
      z__namespace.literal("none")
    ])
  ),
  // JSX specific options
  jsxQuoteStyle: z__namespace.optional(
    z__namespace.union([
      z__namespace.literal("single"),
      z__namespace.literal("double")
    ])
  )
}, {});

// src/Report.ts
var Report_exports = {};
__export(Report_exports, {
  make: () => make3
});
function make3(context) {
  return {
    send(descriptor) {
      if (descriptor == null) return;
      return context.report(descriptor);
    },
    sendOrElse(descriptor, fallback) {
      if (descriptor == null) return fallback();
      return context.report(descriptor);
    }
  };
}

// src/SEL.ts
var SEL_exports = {};
__export(SEL_exports, {
  DISPLAY_NAME_ASSIGNMENT_EXPRESSION: () => DISPLAY_NAME_ASSIGNMENT_EXPRESSION,
  IMPLICIT_RETURN_ARROW_FUNCTION_EXPRESSION: () => IMPLICIT_RETURN_ARROW_FUNCTION_EXPRESSION,
  OBJECT_DESTRUCTURING_VARIABLE_DECLARATOR: () => OBJECT_DESTRUCTURING_VARIABLE_DECLARATOR
});
var IMPLICIT_RETURN_ARROW_FUNCTION_EXPRESSION = "ArrowFunctionExpression[body.type!='BlockStatement']";
var OBJECT_DESTRUCTURING_VARIABLE_DECLARATOR = [
  "VariableDeclarator",
  "[id.type='ObjectPattern']",
  "[init.type='Identifier']"
].join("");
var DISPLAY_NAME_ASSIGNMENT_EXPRESSION = [
  "AssignmentExpression",
  "[operator='=']",
  "[left.type='MemberExpression']",
  "[left.property.name='displayName']"
].join("");

exports.JsxConfig = JsxConfig_exports;
exports.LanguagePreference = LanguagePreference_exports;
exports.RE = RE_exports;
exports.Report = Report_exports;
exports.SEL = SEL_exports;
